Rem Midiplay.bas is the .mid player function for QB64 PD 2024.
Rem   v1.0a first release 05/02/2024
Rem   v1.1a new release 05/05/2024
Rem   v2.0a new release 05/06/2024

' declare variables.
Dim SoundHandle As Long
Dim SoundVolume As Single

' enable midi support.
$Unstable:Midi
$MidiSoundFont:Default

' some .mid soundfonts.
'$MidiSoundFont:"silly.sf2"
'$MidiSoundFont:"steelgit.sf2"

' setup screen.
_Title "Midiplay"
_ScreenMove _Middle

' reset statusbar v2.0a
Screen 0
Width 80, 25
Cls
Locate 25, 1
Color 15, 1
Print Space$(80);
Locate 25, 1
Print "Midiplay "; Date$; " "; Time$;
Locate 24, 1
Color , 0

' init volume
SoundVolume = 1!

' start midi interface.
Color 15
Print "Welcome to Midi player v2.0a."
Color 14
StartLoop:

Rem Call dialog v1.1a
ext$ = "*.wav|*.aiff|*.aifc|*.flac|*.ogg|*.mp3|*.mid|*.it|*.xm|*.s3m|*.mod|*.rad|*.ahx|*.mvl|*.qoa"
Midi$ = _OpenFileDialog$("Open File", "", ext$, "Audio files", 0)

If Midi$ = "" Then Color 7: End
Midi$ = UCase$(Midi$)
X = VerifyPlay(Midi$)
If X = 0 Then
   Print "File not found."
   GoTo StartLoop
End If
If _FileExists(Midi$) = 0 Then
   Print "File not found."
   GoTo StartLoop
End If
X = VerifyPlay(Midi$)
If X Then
   If _FileExists(Midi$) Then
      Color 15
      Print "  Enter <space>=pause/+plus 10/-minus 10/?=length."
      Print "  Press <escape> to exit, <enter> to restart."
      Print "  <up> increase volume by .1 seconds."
      Print "  <down> decrease volume by .1 seconds."
      Print "  <right> Move playback plus 10 seconds."
      Print "  <left> Move playback minus 10 seconds."
      Print "  <ctrl-right> Moves position plus 30 seconds."
      Print "  <ctrl-left> Moves position minus 30 seconds."
      Color 14

      Rem Added v1.1a
      If _MessageBox("MidiPlay", "Load: " + Midi$, "yesno", "question") = 0 Then Color 7: End

      Print "Loading.."
      SoundHandle = _SndOpen(Midi$)
      If SoundHandle = 0 Then
         Print "File not found."
         GoTo StartLoop
      End If
      If SoundHandle Then
         Print "Playing.."
         _SndPlay SoundHandle

         ' reset statusbar v2.0a
         Locate 25, 1
         Color 15, 1
         Print Space$(80);
         Locate 25, 1
         S$ = "Midiplay " + Date$ + " " + Time$ + " - " + Midi$
         If Len(S$) > 77 Then
            S$ = Left$(S$, 77) + "..."
         End If
         S$ = Left$(S$, 80)
         Print S$;
         Locate 24, 1
         Color 14, 0
         _Title "Midiplay " + Midi$

         ' reset to full volume in v2.0a
         SoundVolume = 1!
         _SndVol SoundHandle, SoundVolume

         ' soundplay loop
         Start = Timer
         Do
            ' realtime statusline display.
            Elapsed = Timer - Start
            If Elapsed < 0 Then Elapsed = Elapsed + 86400
            If Elapsed >= 1 Then
               Start = Timer
               X = CsrLin: Y = Pos(0)
               ' store soundfile position
               T& = _SndGetPos(SoundHandle)
               T$ = Str$(T&)
               If InStr(T$, ".") Then
                  T$ = Left$(T$, InStr(T$, ".") + 2)
               End If
               ' store soundfile length
               U& = _SndLen(SoundHandle)
               U$ = Str$(U&)
               If InStr(U$, ".") Then
                  U$ = Left$(U$, InStr(U$, ".") + 2)
               End If
               ' store seconds display
               T$ = " (" + LTrim$(T$) + " of" + U$ + " seconds)"
               Locate 25, 1
               Color 15, 1
               Print Space$(80);
               Locate 25, 1
               S$ = "Midiplay " + Date$ + " " + Time$ + " - " + Midi$
               ' adjust line length
               If Len(S$) > 77 Then
                  S$ = Left$(S$, 77) + "..."
               Else
                  If Len(S$ + T$) <= 80 Then
                     S$ = S$ + T$
                  End If
               End If
               S$ = Left$(S$, 80)
               Print S$;
               Locate X, Y, 1
               Color 14, 0
            End If
            ' continue inkey loop.
            _Limit 50
            x$ = InKey$
            If _SndGetPos(SoundHandle) >= _SndLen(SoundHandle) Then
               Print "Sound stopped."
               _SndStop SoundHandle
               _SndClose SoundHandle
               Print "Press key:"
               Do
                  If InKey$ <> "" Then
                     Exit Do
                  End If
               Loop
               GoTo StartLoop
            End If
            If Len(x$) Then
               ' adjust volume control
               If x$ = Chr$(0) + Chr$(72) Then ' up
                  SoundVolume = SoundVolume + .1!
                  If SoundVolume > 1! Then SoundVolume = 1!
                  _SndVol SoundHandle, SoundVolume
                  S$ = Str$(SoundVolume)
                  If InStr(S$, ".") Then
                     S$ = Left$(S$, InStr(S$, ".") + 1)
                  End If
                  Print "Volume is"; S$; " of 1."
               End If
               If x$ = Chr$(0) + Chr$(80) Then ' down
                  SoundVolume = SoundVolume - .1!
                  If SoundVolume < 0! Then SoundVolume = 0!
                  _SndVol SoundHandle, SoundVolume
                  S$ = Str$(SoundVolume)
                  If InStr(S$, ".") Then
                     S$ = Left$(S$, InStr(S$, ".") + 1)
                  End If
                  Print "Volume is"; S$; " of 1."
               End If

               If x$ = Chr$(27) Then

                  Rem Added v1.1a
                  _MessageBox "MidiPlay", "Press <enter> to quit: ", "info"

                  _SndStop SoundHandle
                  _SndClose SoundHandle
                  Exit Do
               End If
               If x$ = Chr$(13) Then
                  _SndStop SoundHandle
                  _SndClose SoundHandle
                  GoTo StartLoop
               End If
               If x$ = "?" Then
                  S& = _SndLen(SoundHandle)
                  S$ = Str$(S&)
                  If InStr(S$, ".") Then
                     S$ = Left$(S$, InStr(S$, ".") + 2)
                  End If
                  Print "Handle"; SoundHandle; "length is"; S$; " seconds."
                  S$ = Str$(SoundVolume)
                  If InStr(S$, ".") Then
                     S$ = Left$(S$, InStr(S$, ".") + 1)
                  End If
                  Print "Position is"; _SndGetPos(SoundHandle); "seconds. Volume is"; S$; " of 1."
               End If
               If x$ = "+" Or x$ = Chr$(0) + Chr$(77) Then ' right
                  z! = _SndGetPos(SoundHandle)
                  z! = z! + 10!
                  _SndSetPos SoundHandle, z!
                  S& = _SndGetPos(SoundHandle)
                  S$ = Str$(S&)
                  If InStr(S$, ".") Then
                     S$ = Left$(S$, InStr(S$, ".") + 2)
                  End If
                  Print "Position is"; S$; " seconds."
               End If
               If x$ = "-" Or x$ = Chr$(0) + Chr$(75) Then ' left
                  z! = _SndGetPos(SoundHandle)
                  z! = z! - 10!
                  If z! < 0! Then z! = 0!
                  _SndSetPos SoundHandle, z!
                  S& = _SndGetPos(SoundHandle)
                  S$ = Str$(S&)
                  If InStr(S$, ".") Then
                     S$ = Left$(S$, InStr(S$, ".") + 2)
                  End If
                  Print "Position is"; S$; " seconds."
               End If
               If x$ = Chr$(0) + Chr$(116) Then ' ctrl-right
                  z! = _SndGetPos(SoundHandle)
                  z! = z! + 30!
                  _SndSetPos SoundHandle, z!
                  S& = _SndGetPos(SoundHandle)
                  S$ = Str$(S&)
                  If InStr(S$, ".") Then
                     S$ = Left$(S$, InStr(S$, ".") + 2)
                  End If
                  Print "Position is"; S$; " seconds."
               End If
               If x$ = Chr$(0) + Chr$(115) Then ' ctrl-left
                  z! = _SndGetPos(SoundHandle)
                  z! = z! - 30!
                  If z! < 0! Then z! = 0!
                  _SndSetPos SoundHandle, z!
                  S& = _SndGetPos(SoundHandle)
                  S$ = Str$(S&)
                  If InStr(S$, ".") Then
                     S$ = Left$(S$, InStr(S$, ".") + 2)
                  End If
                  Print "Position is"; S$; " seconds."
               End If
               If x$ = " " Then
                  If _SndPaused(SoundHandle) Then
                     Print "Resume play at"; _SndGetPos(SoundHandle); "seconds."
                     _SndPlay SoundHandle
                  Else
                     Print "Pause play at"; _SndGetPos(SoundHandle); "seconds."
                     _SndPause (SoundHandle)
                  End If
               End If
            End If
         Loop
      End If
   End If
End If
Color 7
End

Function VerifyPlay (Inpt$)
   VerifyPlay = 0
   'If ValidFileChar(Inpt$) Then
   '   Exit Function
   'End If
   Var$ = UCase$(Inpt$)
   For X = Len(Var$) To 1 Step -1
      If Mid$(Var$, X, 1) = "." Then
         Var$ = Mid$(Var$, X + 1)
      End If
   Next
   If Len(Var$) Then
      Select Case Var$
         Case "WAV", "AIFF", "AIFC", "FLAC"
            VerifyPlay = -1
         Case "OGG", "MP3", "MID", "IT"
            VerifyPlay = -1
         Case "XM", "S3M", "MOD", "RAD"
            VerifyPlay = -1
         Case "AHX", "HVL", "QOA"
            VerifyPlay = -1
      End Select
   End If
End Function

